//     ======================= AssociativeMemory32s.h ==========================
//     **********************************************************************
//     Author: Yeremeyev Dmitriy Vladimirovich *** mailto:edvmail@mail.ru
//     Last update : 26.09.2009
//     **********************************************************************

#if !defined(_AssociativeMemory32s_H_INCLUDED_)
#define _AssociativeMemory32s_H_INCLUDED_

#include "AssociativeMemory.h"

// Definition of object recognition function
typedef void (*TYPE_RcgAMFunc32S)(CvAM_State aState, long* apData, CvAM_Parameters* apParam, void* args);
typedef TYPE_RcgAMFunc32S CvRcgAMFunc32S;

// Definition of associative memory operation function
typedef bool (*TYPE_AMFunc32S)(CvSize  aRcgMxSize, short* apRcgMx, long* apData,
		                        uint64 aIndex, uint64 aHitCounter,
								int aContainBasesTotal, short aLevel, void* args);
typedef TYPE_AMFunc32S CvAMFunc32S;

// Associative video memory consist of associative bases tree and use input
// image as a key for data access.


// Associative memory class for integer 32-bit data
class CvAssociativeMemory32S {
	void* p;
public:

	CvAssociativeMemory32S(int aMemStorageBlockSize = 0);
	~CvAssociativeMemory32S();
	
	// Creating of associative memory
	//
	// Notes: The parameter aKeyImgSize is depended on sequence 40, 80, 160, 320, 640, 1280... 2^n*10.
	//        Use of this numbers for sizing gives more accuracy in recognition.
	//        The parameter aLevelMax set a maximal level for associative tree.
	//        If set to zero then a maximal level will be computed as optimal for image key size.
	//        aTreeTotal  - total number of independent associative trees.
	//        aClustering - flag of using cluster tree
	void Create(CvSize aKeyImgSize, short aLevelMax = 0,
		        int aTreeTotal = 1, bool aClustering = true);
	// Destroying of associative memory
	void Destroy();

	// Set the active associative tree
	void SetActiveTree(int aTreeIdx);
	// Removing all data out from associative tree
	void ClearTreeData();

	// Set an image for processing
	void SetImage(const void* apSrcImg, const void* apObjMaskImg = NULL);

	// Reading from associative memory cell (associative base)
	//
	// Notes: aInterestArea - interest area for object searching;
	//        apObjRect     - rectangle where object found;
	//        appData       - pointer to the data of associative cell;
	//        apIndex       - unique index of associative base;
	//        apHitCounter  - number of hitting to associative base;
	//        apSimilarity  - similarity of the interest area to an object (0 ... 1);
	//        aTotalSearch  - flag of the total search until end level of associative memory.
	bool Read(CvRect aInterestArea, CvRect* apObjRect, long** appData,
		      uint64* apIndex = NULL, uint64* apHitCounter = NULL,
			  double* apSimilarity = NULL, bool aTotalSearch = false);

	// Writing to associative memory cell
	void Write(CvRect aInterestArea, long* apData, bool aDeepLearning = true);

	// Object recognition
	void ObjectRecognition(CvRcgAMFunc32S RcgAMFunc, void *args = NULL);
	void ObjectTracking(CvRcgAMFunc32S RcgAMFunc, void *args = NULL,
		                bool   aTrainingDuringTracking = true,
		                double aSimilarityUpTrd = cAM_SimilarityUpTrd,
						double aSimilarityDnTrd = cAM_SimilarityDnTrd);

	// Optimization of associative tree
	void OptimizeAssociativeTree();
	// Restart time counter of optimization event
	void RestartTimeForOptimization();
	
	// Get size of packed recognition data
	long GetPackedDataSize();
	// Writing a packed recognition data to memory
	void WritePackedData(void* apMem);
	// Reading a packed recognition data from memory
	void ReadPackedData(void* apMem);
	
	// Saving of recognition data
	bool Save(char* aFileName, bool aSaveWithoutOptimization = false);
	// Loading of recognition data
	bool Load(char* aFileName);

	// The estimation of an opportunity for training
	double EstimateOpportunityForTraining(CvRect aInterestArea);

	// Perform "AMFunc" for each associative base
	bool DoForEach(CvAMFunc32S AMFunc, void *args);
	// Perform "RcgMCFunc" for each node of associative bases
	bool DoForEachNode(CvRcgMCFunc16S RcgMCFunc, void *args);
	// Get base level for nodes
	short GetBaseNodeLevel(short aABaseLevel);
	// Get total number of clusters
	long GetClustersTotal();
	// Get total number of node levels
	short GetNodeTotalLevels();
	
	// Get current index of associative base
	uint64 GetCurIndex();
	// Get total number of associative bases
	long   GetTotalABases();
	// Get total number of memory levels
	short  GetTotalLevels();
	// Get read-write counter
	uint64 GetWrRdCounter();
	// Get size of key image of associative memory
	CvSize GetKeyImageSize();
	// Get base key size 
	CvSize GetBaseKeySize();
	// Set the value of parameter 
	void SetParam(CvAM_ParamType aParam, double aValue);
};

#endif // !defined(_AssociativeMemory32s_H_INCLUDED_)
